﻿#region Declaración de Objetos utilizados
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
//using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
//using System.Windows.Shapes;
using System.IO;
using Microsoft.Kinect;
using Emgu.CV;
using Emgu.CV.Structure;
using System.Drawing;
using System.Drawing.Imaging;
using System.Net.Sockets;
#endregion

namespace AjustarSeguimiento
{
    /// <summary>
    /// Lógica de interacción para MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
#region Declaración de Constantes y Variables de la clase
        const int NUM_MARCADORES = 4; //0 Ningun Marcador, 1 M1,...
        const int NUM_PARAM = 4; //2 de H y 2 de S & ninguno de V

        /*************************Variables Globales******************************/
        /// <summary>
        /// Pinta de color rojo el marcador que no está siendo utilizado
        /// </summary>
        System.Windows.Media.Color ColorMarcadorActivado = System.Windows.Media.Color.FromRgb((byte)0, (byte)255, (byte)0);
        /// <summary>
        /// Pinta de color verde el marcador que está siendo utilizado
        /// </summary>
        System.Windows.Media.Color ColorMarcadorDesactivado = System.Windows.Media.Color.FromRgb((byte)255, (byte)0, (byte)0);
        /*Definición de subprocesos ciclicos*/
        /// <summary>
        /// Definición de Subproceso ciclico de Búsqueda del marcador individualmente
        /// </summary>
        System.Windows.Forms.Timer marcador = new System.Windows.Forms.Timer();
        /// <summary>
        /// Definición de Subproceso ciclico del Marcador 1
        /// </summary>
        System.Windows.Forms.Timer marcador1 = new System.Windows.Forms.Timer();
        /// <summary>
        /// Definición de Subproceso ciclico del Marcador 2
        /// </summary>
        System.Windows.Forms.Timer marcador2 = new System.Windows.Forms.Timer();       
        /// <summary>
        /// Definición de Subproceso ciclico del Marcador 3
        /// </summary>
        System.Windows.Forms.Timer marcador3 = new System.Windows.Forms.Timer();

        System.Windows.Forms.Timer envio = new System.Windows.Forms.Timer();
        /*Variables Utilizadas por Kinect*/
        /// <summary>
        /// sensor Kinect activo
        /// </summary>
        private KinectSensor sensor;
        /// <summary>
        /// Estructura donde almacenar los pixeles de profundidad
        /// </summary>
        private DepthImagePixel[] depthPixels;
        /// <summary>
        /// Estructura donde almacenar los pixeles de color
        /// </summary>
        private byte[] colorPixels;
        /// <summary>
        /// Matriz de puntos de profundidad para recibir los datos
        /// </summary>
        DepthImagePoint[] mappedColorLocations = new DepthImagePoint[307200];
        /// <summary>
        /// Puntos en el espacio Esqueleto x y z del Marcador
        /// </summary>
        SkeletonPoint PuntoEnSkeleto;
        /*Declaración de imagenes Emgucv*/
        /// <summary>
        /// Imagen recibida por Kinect convertida a Bitmap
        /// </summary>
        Image<Bgr, Byte> ImageFrame = new Image<Bgr, Byte>(640, 480);
        /// <summary>
        /// copia de la región de interes de ImageFrame utilizada por el ciclio marcador1
        /// </summary>
        Image<Bgr, Byte> ImageFrame1 = new Image<Bgr, Byte>(640, 480);
        /// <summary>
        /// copia de la región de interes de ImageFrame utilizada por el ciclio marcador2
        /// </summary>
        Image<Bgr, Byte> ImageFrame2 = new Image<Bgr, Byte>(640, 480);
        /// <summary>
        /// copia de la región de interes de ImageFrame utilizada por el ciclio marcador3
        /// </summary>
        Image<Bgr, Byte> ImageFrame3 = new Image<Bgr, Byte>(640, 480);
        /// <summary>
        /// ROI de la imagen para el marcador1
        /// </summary>
        Rectangle Rectangulo1 = new Rectangle(100, 200, 80, 60);
        /// <summary>
        /// ROI de la imagen para el marcador2
        /// </summary>
        Rectangle Rectangulo2 = new Rectangle(200, 200, 80, 60);
        /// <summary>
        /// ROI de la imagen para el marcador3
        /// </summary>
        Rectangle Rectangulo3 = new Rectangle(300, 200, 80, 60);
        /// <summary>
        /// Activar el seguimiento del los tres marcadores
        /// </summary>
        bool seguimiento = false;

        int xROI = 0;
        int yROI = 0;
        int wROI = 640;
        int hROI = 480;
        int minwROI = 100;
        int minhROI = 75;
        int bwROI = 20;
        int bhROI = 15;
        int fwROI = 20;
        int fhROI = 15;        
        /**************************Memoria Compartida*****************************/
        /// <summary>
        /// Sincroniza la escritura de los valores de los Marcadores
        /// </summary>
        private object MarcLock = new Object();
        /// <summary>
        /// Marcador que está siguiendo el programa
        /// </summary>
        int MaAct = 0; //0 ninguno
        /// <summary>
        /// Array bidireccional donde se almacenan el valor de los Marcadores
        /// </summary>
        int[,] ValMar = new int[NUM_MARCADORES, NUM_PARAM];
        /// <summary>
        /// Determina si el valor del marcador debe cambiar al mover la barra deslizadora
        /// </summary>
        bool chValMac = true;
        /// <summary>
        /// Identificador que indica en que casilla Imagen esta representada la imagen a color. 0 -> posición Principal;
        /// 1 -> posición Secundaria 1; 2 -> Posición Secundaria 2
        /// </summary>
        int imagenColor = 0; //Posición Principal al inicio
        /// <summary>
        /// Identificador que indica en que casilla Imagen esta representada la imagen binarizada. 0 -> posición Principal;
        /// 1 -> posición Secundaria 1; 2 -> Posición Secundaria 2
        /// </summary>
        int imagenBinaria = 1; //Posición Secundaria 1 al inicio
        /// <summary>
        /// Identificador que indica en que casilla Imagen esta representada la imagen erosionada. 0 -> posición Principal;
        /// 1 -> posición Secundaria 1; 2 -> Posición Secundaria 2
        /// </summary>
        int imagenErosionada = 2; //Posición Secunadaria 2 al inicio
        /// <summary>
        /// Identificador que indica en que casilla Imagen esta representada la imagen a color del Marcador 1. 0 -> posición Principal;
        /// 1 -> posición Secundaria 1; 2 -> Posición Secundaria 2
        /// </summary>
        int imagenColor1 = 0; //Posición Principal al inicio
        /// <summary>
        /// Identificador que indica en que casilla Imagen esta representada la imagen a color del Marcador 2. 0 -> posición Principal;
        /// 1 -> posición Secundaria 1; 2 -> Posición Secundaria 2
        /// </summary>
        int imagenColor2 = 1; //Posición Principal al inicio
        /// <summary>
        /// Identificador que indica en que casilla Imagen esta representada la imagen a color del Marcador 3. 0 -> posición Principal;
        /// 1 -> posición Secundaria 1; 2 -> Posición Secundaria 2
        /// </summary>
        int imagenColor3 = 2; //Posición Principal al inicio
        /// <summary>
        /// Desplazamiento sobre el pixel x necesario para realizar un buen Mapeo
        /// </summary>
        int despX = 0;
        /// <summary>
        /// Desplazamiento sobre el pixel y necesario para realizar un buen Mapeo
        /// </summary>
        int despY = 3;
        /// <summary>
        /// ruta en la que se graban los pixeles y puntos de un marcador inividual
        /// </summary>
        string rutaTrayectoria;
        /// <summary>
        /// permite la grabación de los pixeles y puntos de un marcador inividual
        /// </summary>
        bool grabar = false;
        #endregion
#region inicio y fin de programa
        public MainWindow()
        {
            //Iniciar los componentes xaml
            InitializeComponent();
            /*Manejadores de los eventos ciclicos de búsqueda de los marcadores*/
            //Marcador individual
            marcador.Tick += new EventHandler(marcador_Tick);
            marcador.Interval = 34; //34 -> 30FPS
            //Marcador 1
            marcador1.Tick += new EventHandler(marcador1_Tick);
           // marcador1.Interval = 34; //34 -> 30FPS
            //Marcador 2
            marcador2.Tick += new EventHandler(marcador2_Tick);
            //marcador2.Interval = 34; //34 -> 30FPS
            //Marcador 3
            marcador3.Tick += new EventHandler(marcador3_Tick);
            //marcador3.Interval = 34; //34 -> 30FPS
            envio.Tick += new EventHandler(envio_Tick);
            envio.Interval = 30;
            //Manejador del evento carga de programa
            Loaded += new RoutedEventHandler(AjustarSeguimiento_Loaded);
            //Manejador del evento que se produce cuando la ventana esta apunto de cerrarse
            Closed += new EventHandler(AjustarSeguimiento_Closed);
        }

             
        /// <summary>
        /// Evento Inicio de Ejecución
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void AjustarSeguimiento_Loaded(object sender, RoutedEventArgs e)
        {
            //Buscar Sensor Kinect y activarlo
            foreach (var potentialSensor in KinectSensor.KinectSensors)
            {
                if (potentialSensor.Status == KinectStatus.Connected)
                {
                    this.sensor = potentialSensor;
                    break;
                }
            }
            //Siempre que haya un sensor activo
            if (null != this.sensor)
            {
                // Permitir recibir stream de color 
                this.sensor.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30);

                // Permitir recibir stream de profundidad
                this.sensor.DepthStream.Enable(DepthImageFormat.Resolution640x480Fps30);

                //Rango cercano de profundidad (mayor precisión)
                this.sensor.DepthStream.Range = DepthRange.Near;

                // Reservar espacio donde poner los pixeles de profundidad recibidos
                this.depthPixels = new DepthImagePixel[this.sensor.DepthStream.FramePixelDataLength];

                // Reservar espacio donde poner los pixeles de color recibidos
                this.colorPixels = new byte[this.sensor.ColorStream.FramePixelDataLength];

                //Manejador del evento ColorFrame listo
                this.sensor.ColorFrameReady += sensor_ColorFrameReady;
                //Manejador del evento DepthFrame listo
                this.sensor.DepthFrameReady += sensor_DepthFrameReady;

                // Start the sensor!
                try
                {
                    this.sensor.Start();
                }
                catch (InvalidOperationException)
                {                    
                    this.sensor = null;                    
                }
                marcador.Enabled = true;
                marcador.Start();                
            }
        }
        /// <summary>
        /// evento Fin de Ejecución
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void AjustarSeguimiento_Closed(object sender, EventArgs e)
        {
            //Parar el subproceso marcador
            marcador.Stop();
            //Matar el subproceso marcador
            marcador.Dispose();
            //Parar el subproceso marcador1
            marcador1.Stop();
            //Matar el subproceso marcador1
            marcador1.Dispose();
            //Parar el subproceso marcador2
            marcador2.Stop();
            //Matar el subproceso marcador2
            marcador2.Dispose();
            //Parar el subproceso marcador3
            marcador3.Stop();
            //Matar el subproceso marcador3
            marcador3.Dispose();
            //Desactivar el sensor
            if (null != this.sensor)
            {
                this.sensor.Stop();
            }
        }
        #endregion
#region eventos de kinect
        /// <summary>
        /// Codigo ejecutado cuando hay un Frame de Color Listo
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void sensor_ColorFrameReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            //EVENTO CÁMARA COLOR
            using (ColorImageFrame colorFrame = e.OpenColorImageFrame())
            {
                if (colorFrame != null)
                {
                    //Conversion del Color Frame de kinect en formato BitmapSource a Bitmap
                    Bitmap bmpFrame = ColorImageFrameToBitmap(colorFrame);
                    //ImageFrame será la imagen a tratar 
                    ImageFrame = new Image<Bgr, Byte>(bmpFrame);
                    //Seguimiento de los tres marcadores
                    if (seguimiento)
                    {
                        //Reset la ROI de w
                        //ImageFrame.ROI = System.Drawing.Rectangle.Empty;
                        //Clonar la imagenFrame
                        ImageFrame1 = ImageFrame.Clone();
                        ImageFrame2 = ImageFrame.Clone();
                        ImageFrame3 = ImageFrame.Clone();
                        //Establecer la ROI para el marcador 1
                        ImageFrame1.ROI = Rectangulo1;
                        //Representación de la imagen a color de la ROI 1
                        switch (imagenColor1)
                        {
                            case 0:
                                ImagenPrincipal.Source = BitmapSourceConvert.ToBitmapSource(ImageFrame1);
                                break;
                            case 1:
                                ImagenSecundaria1.Source = BitmapSourceConvert.ToBitmapSource(ImageFrame1);
                                break;
                            case 2:
                                ImagenSecundaria2.Source = BitmapSourceConvert.ToBitmapSource(ImageFrame1);
                                break;
                        }
                        //Establecer la ROI para el marcador 2
                        ImageFrame2.ROI = Rectangulo2;
                        //Representación de la imagen a color de la ROI 2
                        switch (imagenColor2)
                        {
                            case 0:
                                ImagenPrincipal.Source = BitmapSourceConvert.ToBitmapSource(ImageFrame2);
                                break;
                            case 1:
                                ImagenSecundaria1.Source = BitmapSourceConvert.ToBitmapSource(ImageFrame2);
                                break;
                            case 2:
                                ImagenSecundaria2.Source = BitmapSourceConvert.ToBitmapSource(ImageFrame2);
                                break;
                        }
                        //Establecer la ROI para el marcador 1
                        ImageFrame3.ROI = Rectangulo3;
                        //Representación de la imagen a color de la ROI 3
                        switch (imagenColor3)
                        {
                            case 0:
                                ImagenPrincipal.Source = BitmapSourceConvert.ToBitmapSource(ImageFrame3);
                                break;
                            case 1:
                                ImagenSecundaria1.Source = BitmapSourceConvert.ToBitmapSource(ImageFrame3);
                                break;
                            case 2:
                                ImagenSecundaria2.Source = BitmapSourceConvert.ToBitmapSource(ImageFrame3);
                                break;
                        }
                    }
                    //seguimiento individual de un marcador
                    else
                    {
                        if (permisoROI.IsChecked == false)
                        {
                            //Establecer la región de interés
                            ImageFrame.ROI = new Rectangle(xROI, yROI, wROI, hROI);
                        }
                        else
                        {
                            ImageFrame.ROI = new Rectangle(250, 190, 140, 105);
                        }
                        //Representar la Imagen a color en la casilla Imagen que le corresponde                    
                        switch (imagenColor)
                        {
                            case 0:
                                ImagenPrincipal.Source = BitmapSourceConvert.ToBitmapSource(ImageFrame);
                                break;
                            case 1:
                                ImagenSecundaria1.Source = BitmapSourceConvert.ToBitmapSource(ImageFrame);
                                break;
                            case 2:
                                ImagenSecundaria2.Source = BitmapSourceConvert.ToBitmapSource(ImageFrame);
                                break;
                        }                                                
                    }
                }
            }
        }
        /// <summary>
        /// Codigo ejecutado cuando hay un Frame Profundidad Listo
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void sensor_DepthFrameReady(object sender, DepthImageFrameReadyEventArgs e)
        {
            //EVENTO CÁMARA DE PROFUNDIDAZ
            using (DepthImageFrame depthFrame = e.OpenDepthImageFrame())
            {
                if (depthFrame != null)
                {
                    //Copy the pre-pixel depth data to a pre-allocated array
                    depthFrame.CopyDepthImagePixelDataTo(depthPixels);
                    //Mapear los pixeles de la imagen con los datos de profundidad
                    this.sensor.CoordinateMapper.MapColorFrameToDepthFrame(this.sensor.ColorStream.Format, this.sensor.DepthStream.Format, depthPixels, mappedColorLocations);
                }
            }
        }
        #endregion
#region ciclo de búsqueda de marcadores
        /// <summary>
        /// Código ejecutado en cada ciclo de búsqueda del marcador individual
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void marcador_Tick(object sender, EventArgs e)
        {
            int[] paramMarcador = new int[NUM_PARAM]; //Parametros del color a buscar

            //los parametros del color a seguir serán los del marcador activo
            lock (MarcLock)
            {
                for (int i = 0; i < NUM_PARAM; i++)
                {
                    paramMarcador[i] = ValMar[MaAct, i];
                }
            }
            //Convierte la imagen rgb a una imagen binaria donde los pixeles blancos serán el color del marcador
            Image<Gray, Byte> ImageFrameDetection = cvAndHsvImage(
                ImageFrame, paramMarcador[0], paramMarcador[1], paramMarcador[2], paramMarcador[3]);
            //Se localiza la posición del centro de gravedad del conjunto de pixeles blancos (color marcador)
            RecDetection(ImageFrameDetection);
        }
        /// <summary>
        /// Código ejecutado en cada ciclo de búsqueda del marcador 1
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void marcador1_Tick(object sender, EventArgs e)
        {
            //Convierte la imagen rgb a una imagen binaria donde los pixeles blancos serán el color del marcador
            Image<Gray, Byte> ImageFrameDetection = cvAndHsvImage(
                ImageFrame1, ValMar[1, 0], ValMar[1, 1], ValMar[1, 2], ValMar[1,3]);
            //Se localiza la posición del centro de gravedad del conjunto de pixeles blancos (color marcador)
            RecDetection1(ImageFrameDetection);
        }
        /// <summary>
        /// Código ejecutado en cada ciclo de búsqueda del marcador 2
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void marcador2_Tick(object sender, EventArgs e)
        {
            //Convierte la imagen rgb a una imagen binaria donde los pixeles blancos serán el color del marcador
            Image<Gray, Byte> ImageFrameDetection = cvAndHsvImage(
                ImageFrame2, ValMar[2, 0], ValMar[2, 1], ValMar[2, 2], ValMar[2, 3]);
            //Se localiza la posición del centro de gravedad del conjunto de pixeles blancos (color marcador)
            RecDetection2(ImageFrameDetection);
        }
        /// <summary>
        /// Código ejecutado en cada ciclo de búsqueda del marcador 3
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void marcador3_Tick(object sender, EventArgs e)
        {
            //Convierte la imagen rgb a una imagen binaria donde los pixeles blancos serán el color del marcador
            Image<Gray, Byte> ImageFrameDetection = cvAndHsvImage(
                ImageFrame3, ValMar[3, 0], ValMar[3, 1], ValMar[3, 2], ValMar[3, 3]);
            //Se localiza la posición del centro de gravedad del conjunto de pixeles blancos (color marcador)
            RecDetection3(ImageFrameDetection);
        }               
        #endregion
#region Funciones para binarizar imagen Color
        /// <summary>
        /// convierte la imagen a color a una imagen Binarizada según los parametros de color
        /// </summary>
        /// <param name="imgFrame">Imagen en formato Bitmap ha binarizar</param>
        /// <param name="L1">parametro HL</param>
        /// <param name="H1">parametro HH</param>
        /// <param name="L2">parametro SL</param>
        /// <param name="H2">parametro SH</param>
        private Image<Gray, byte> cvAndHsvImage(Image<Bgr, byte> imgFrame, int L1, int H1, int L2, int H2)
        {
            Image<Hsv, Byte> hsvImage = imgFrame.Convert<Hsv, Byte>();
            Image<Gray, Byte> ResultImage = new Image<Gray, Byte>(hsvImage.Width, hsvImage.Height);
            Image<Gray, Byte> ResultImage2 = new Image<Gray, Byte>(hsvImage.Width, hsvImage.Height);
            Image<Gray, Byte> ResultImage3 = new Image<Gray, Byte>(hsvImage.Width, hsvImage.Height);

            //Convertir la imagen HSV a imagen H en grises
            Image<Gray, Byte> ResultImageH = inRangeImage(hsvImage, L1, H1, 0);
            
            //Convertir la imagen HSV a imagen S en grises
            Image<Gray, Byte> ResultImageS = inRangeImage(hsvImage, L2, H2, 1);           
            //Binarizar la imagen                        
            CvInvoke.cvAnd(ResultImageH, ResultImageS, ResultImage, System.IntPtr.Zero);
            
            //Erosionar la imagen 
            CvInvoke.cvErode(ResultImage, ResultImage2, (IntPtr)null, 1);
            //Dilatar posteriormente
            CvInvoke.cvDilate(ResultImage2, ResultImage3, (IntPtr)null, 1);
            
            //Solamente cuando se sigue un único marcador representar estas imagenes
            if (!seguimiento)
            {
                //Representar la Imagen binarizada en la casilla Imagen que le corresponde
                switch (imagenBinaria)
                {
                    case 0:
                        ImagenPrincipal.Source = BitmapSourceConvert.ToBitmapSource(ResultImage);
                        break;
                    case 1:
                        ImagenSecundaria1.Source = BitmapSourceConvert.ToBitmapSource(ResultImage);
                        break;
                    case 2:
                        ImagenSecundaria2.Source = BitmapSourceConvert.ToBitmapSource(ResultImage);
                        break;
                }
                
                //Representar la Imagen erosionada y dilatada en la casilla Imagen que le corresponde
                switch (imagenErosionada)
                {
                    case 0:
                        ImagenPrincipal.Source = BitmapSourceConvert.ToBitmapSource(ResultImage3);
                        break;
                    case 1:
                        ImagenSecundaria1.Source = BitmapSourceConvert.ToBitmapSource(ResultImage3);
                        break;
                    case 2:
                        ImagenSecundaria2.Source = BitmapSourceConvert.ToBitmapSource(ResultImage3);
                        break;
                }
            }
            return ResultImage3;
        }
        /// <summary>
        /// Convierte una imagen HSV a una imagen gris según los rangos establecidos
        /// </summary>
        /// <param name="hsvImage">Imagen Hsv a convertir</param>
        /// <param name="Lo">Rango inferior</param>
        /// <param name="Hi">Rango superior</param>
        /// <param name="con">0 -> h 1 -> s 2 -> v</param>
        private Image<Gray, Byte> inRangeImage(Image<Hsv, Byte> hsvImage, int Lo, int Hi, int con)
        {
            Image<Gray, Byte> ResultImage = new Image<Gray, Byte>(hsvImage.Width, hsvImage.Height);
            //Generación de Rango inferior
            Image<Gray, Byte> IlowCh = new Image<Gray, Byte>(hsvImage.Width, hsvImage.Height, new Gray(Lo));
            //Generación de Rango superior
            Image<Gray, Byte> IHiCh = new Image<Gray, Byte>(hsvImage.Width, hsvImage.Height, new Gray(Hi));
            //Imagen binaria (Result Imagen) generada entre los rangos inferior y superior 
            CvInvoke.cvInRange(hsvImage[con], IlowCh, IHiCh, ResultImage);
            return ResultImage;
        }
        #endregion
#region Funciones para Buscar Marcador
        /// <summary>
        /// Localización del centro de gravedad del blob blanco de mayor tamaño 
        /// en el seguimient de marcadores individuales
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void RecDetection(Image<Gray, byte> img)
        {
            Image<Gray, Byte> imgForContour = new Image<Gray, byte>(img.Width, img.Height);
            CvInvoke.cvCopy(img, imgForContour, System.IntPtr.Zero);
            IntPtr storage = CvInvoke.cvCreateMemStorage(0);
            IntPtr contour = new IntPtr(); //Puntero al primer contorno encontrado
            //Encontrar Contorno           
            int numContornos = CvInvoke.cvFindContours(
                imgForContour,
                storage,
                ref contour,
                System.Runtime.InteropServices.Marshal.SizeOf(typeof(MCvContour)),
                Emgu.CV.CvEnum.RETR_TYPE.CV_RETR_EXTERNAL,
                Emgu.CV.CvEnum.CHAIN_APPROX_METHOD.CV_CHAIN_APPROX_NONE, new System.Drawing.Point(xROI, yROI));
            //Representar valores Wroi e Hroi
            numwroi.Text = wROI.ToString();
            numhroi.Text = hROI.ToString();                                     
            //Es necesario un punto seq para que lo utilice el método cvBoundingRect
            Seq<System.Drawing.Point> seq = new Seq<System.Drawing.Point>(contour, null);
            for (; seq != null && seq.Ptr.ToInt64() != 0; seq = seq.HNext)
            {
                //Crear rectángulo en el contorno encontrado             
                System.Drawing.Rectangle bndRec = CvInvoke.cvBoundingRect(contour, 1);
                //ancho.Text = bndRec.Width.ToString();
                //Imprimir en ventana el punto central del rectángulo
                pixelX.Text = (bndRec.X + bndRec.Width / 2).ToString();
                pixelY.Text = (bndRec.Y + bndRec.Height / 2).ToString();
                //Calcular el pixel donde se encuentra el centro del contorno calculado                        
                int result = ((bndRec.Y + despY + bndRec.Width / 2) * 640) + (bndRec.X + despX + bndRec.Width / 2);
                //Garantizar que en el momento del mapeo no se salgan de los límites de la matriz
                if (result < (640 * 480))
                {                    
                    //Localizar el punto en el espacio esqueleto que corresponde a ese pixel
                    PuntoEnSkeleto = this.sensor.CoordinateMapper.MapDepthPointToSkeletonPoint(this.sensor.DepthStream.Format, mappedColorLocations[result]);
                    //Representar puntos de esqueleto solamente cuando existan
                    if (KinectSensor.IsKnownPoint(PuntoEnSkeleto))
                    {
                        float valorz = (float)1.34 - PuntoEnSkeleto.Z;
                        puntoX.Text = PuntoEnSkeleto.X.ToString();
                        puntoY.Text = PuntoEnSkeleto.Y.ToString();
                        puntoZ.Text = PuntoEnSkeleto.Z.ToString();                        
                    }
                    //Grabar coordenadas pixel x,y del marcador activo + posicion x,y,z mapeado a puntos de esqueleto
                    if (grabar)
                    {
                        string datosString = (bndRec.X + bndRec.Width / 2).ToString() + ";" + (bndRec.Y + bndRec.Height / 2).ToString() + ";" + puntoX.Text + ";" + puntoY.Text + ";" + puntoZ.Text;
                        using (System.IO.StreamWriter file = new System.IO.StreamWriter(rutaTrayectoria, true))
                        {
                            file.WriteLine(datosString);
                        }
                    }                    
                    
                    //Establecer el nuevo xROI
                    if (((bndRec.X + bndRec.Width / 2) - wROI / 2) >= 0) xROI = (bndRec.X + bndRec.Width / 2) - wROI / 2;
                    else xROI = 0;
                    //Establecer el nuevo yROI
                    if (((bndRec.Y + bndRec.Height / 2) - hROI / 2) >= 0) yROI = (bndRec.Y + bndRec.Height / 2) - hROI / 2;
                    else yROI = 0;
                    //Si no está reducida la región al mínimo reducirla (focalizar la region)
                    if (wROI > minwROI)
                    {
                        wROI = wROI - fwROI;
                        hROI = hROI - fhROI;
                        ancho.Text = "incor";
                    }
                    else
                    {
                        ancho.Text = "Corr";
                    }                                        
                }
            }
            //Ampliar region cuando no se encuentra ningun contorno (buscar el marcador)
            if (seq != null && seq.Ptr.ToInt64() == 0)
            {
                if (wROI < 640)
                {
                    if (xROI - (bwROI / 2) >= 0) xROI = xROI - (bwROI / 2);
                    else xROI = 0;
                    wROI = wROI + bwROI;
                }

                if (hROI < 480)
                {
                    if (yROI - (bhROI / 2) >= 0) yROI = yROI - (bhROI / 2);
                    else yROI = 0;
                    hROI = hROI + bhROI;
                }
            }
        }
        /// <summary>
        /// Localización del centro de gravedad del blob blanco de mayor tamaño 
        /// en el seguimiento del marcador 1
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void RecDetection1(Image<Gray, byte> img)
        {
            Image<Gray, Byte> imgForContour = new Image<Gray, byte>(img.Width, img.Height);
            CvInvoke.cvCopy(img, imgForContour, System.IntPtr.Zero);
            IntPtr storage = CvInvoke.cvCreateMemStorage(0);
            IntPtr contour = new IntPtr(); //Puntero al primer contorno encontrado
            //Encontrar Contorno           
            int numContornos = CvInvoke.cvFindContours(
                imgForContour,
                storage,
                ref contour,
                System.Runtime.InteropServices.Marshal.SizeOf(typeof(MCvContour)),
                Emgu.CV.CvEnum.RETR_TYPE.CV_RETR_EXTERNAL,
                Emgu.CV.CvEnum.CHAIN_APPROX_METHOD.CV_CHAIN_APPROX_NONE, new System.Drawing.Point(Rectangulo1.X, Rectangulo1.Y));
            //Representar valores Wroi e Hroi
            numwroi.Text = Rectangulo1.Width.ToString();
            numhroi.Text = Rectangulo1.Height.ToString(); 
            //Es necesario un punto seq para que lo utilice el método cvBoundingRect
            Seq<System.Drawing.Point> seq = new Seq<System.Drawing.Point>(contour, null);
            for (; seq != null && seq.Ptr.ToInt64() != 0; seq = seq.HNext)
            {
                //Crear rectángulo en el contorno encontrado             
                System.Drawing.Rectangle bndRec = CvInvoke.cvBoundingRect(contour, 1);
                //Calcular el pixel donde se encuentra el centro del contorno calculado                        
                int result = ((bndRec.Y + despY + bndRec.Width / 2) * 640) + (bndRec.X + despX + bndRec.Width / 2);
                //Garantizar que en el momento del mapeo no se salgan de los límites de la matriz
                if (result < (640 * 480))
                {
                    //Localizar el punto en el espacio esqueleto que corresponde a ese pixel
                    PuntoEnSkeleto = this.sensor.CoordinateMapper.MapDepthPointToSkeletonPoint(this.sensor.DepthStream.Format, mappedColorLocations[result]);
                    if (KinectSensor.IsKnownPoint(PuntoEnSkeleto))
                    {
                        guardarposicion(2, PuntoEnSkeleto);
                        m1x.Text = PuntoEnSkeleto.X.ToString();
                        m1y.Text = PuntoEnSkeleto.Y.ToString();
                        m1z.Text = PuntoEnSkeleto.Z.ToString();
                    }
                    /*Establezco el nuevo ROI */
                    //Establecer el nuevo xROI
                    if (((bndRec.X + bndRec.Width / 2) - Rectangulo1.Width / 2) >= 0) Rectangulo1.X = (bndRec.X + bndRec.Width / 2) - Rectangulo1.Width / 2;
                    else Rectangulo1.X = 0;
                    //Establecer el nuevo yROI
                    if (((bndRec.Y + bndRec.Height / 2) - Rectangulo1.Height / 2) >= 0) Rectangulo1.Y = (bndRec.Y + bndRec.Height / 2) - Rectangulo1.Height / 2;
                    else Rectangulo1.Y = 0;
                    //Si no está reducida la región al mínimo reducirla (focalizar la region)
                    if (Rectangulo1.Width > minwROI)
                    {
                        Rectangulo1.Width = Rectangulo1.Width - fwROI;
                        Rectangulo1.Height = Rectangulo1.Height - fhROI;
                        
                    }
                    //el valor encontrado es correcto
                    else
                    {
                        correcto1.Fill = new System.Windows.Media.SolidColorBrush(ColorMarcadorActivado);
                        
                        
                    }
                }
            }
            //Ampliar region cuando no se encuentra ningun contorno (buscar el marcador)
            if (seq != null && seq.Ptr.ToInt64() == 0)
            {
                correcto1.Fill = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);
                if (Rectangulo1.Width < 640)
                {
                    if (xROI - (bwROI / 2) >= 0) Rectangulo1.X = Rectangulo1.X - (bwROI / 2);
                    else Rectangulo1.X = 0;
                    Rectangulo1.Width = Rectangulo1.Width + bwROI;
                }

                if (Rectangulo1.Height < 480)
                {
                    if (Rectangulo1.Y - (bhROI / 2) >= 0) Rectangulo1.Y = Rectangulo1.Y - (bhROI / 2);
                    else Rectangulo1.Y = 0;
                    Rectangulo1.Height = Rectangulo1.Height + bhROI;
                }
            }
        }        
        /// <summary>
        /// Localización del centro de gravedad del blob blanco de mayor tamaño 
        /// en el seguimiento del marcador 2
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void RecDetection2(Image<Gray, byte> img)
        {
            Image<Gray, Byte> imgForContour = new Image<Gray, byte>(img.Width, img.Height);
            CvInvoke.cvCopy(img, imgForContour, System.IntPtr.Zero);
            IntPtr storage = CvInvoke.cvCreateMemStorage(0);
            IntPtr contour = new IntPtr(); //Puntero al primer contorno encontrado
            //Encontrar Contorno           
            int numContornos = CvInvoke.cvFindContours(
                imgForContour,
                storage,
                ref contour,
                System.Runtime.InteropServices.Marshal.SizeOf(typeof(MCvContour)),
                Emgu.CV.CvEnum.RETR_TYPE.CV_RETR_EXTERNAL,
                Emgu.CV.CvEnum.CHAIN_APPROX_METHOD.CV_CHAIN_APPROX_NONE, new System.Drawing.Point(Rectangulo2.X, Rectangulo2.Y));
            //Es necesario un punto seq para que lo utilice el método cvBoundingRect
            Seq<System.Drawing.Point> seq = new Seq<System.Drawing.Point>(contour, null);
            for (; seq != null && seq.Ptr.ToInt64() != 0; seq = seq.HNext)
            {
                //Crear rectángulo en el contorno encontrado             
                System.Drawing.Rectangle bndRec = CvInvoke.cvBoundingRect(contour, 1);
                //Calcular el pixel donde se encuentra el centro del contorno calculado                        
                int result = ((bndRec.Y + despY + bndRec.Width / 2) * 640) + (bndRec.X + despX + bndRec.Width / 2);
                //Garantizar que en el momento del mapeo no se salgan de los límites de la matriz
                if (result < (640 * 480))
                {
                    //Localizar el punto en el espacio esqueleto que corresponde a ese pixel
                    PuntoEnSkeleto = this.sensor.CoordinateMapper.MapDepthPointToSkeletonPoint(this.sensor.DepthStream.Format, mappedColorLocations[result]);
                    if (KinectSensor.IsKnownPoint(PuntoEnSkeleto))
                    {
                        guardarposicion(3, PuntoEnSkeleto);
                        m2x.Text = PuntoEnSkeleto.X.ToString();
                        m2y.Text = PuntoEnSkeleto.Y.ToString();
                        m2z.Text = PuntoEnSkeleto.Z.ToString();
                    }
                    /*Establezco el nuevo ROI */
                    //Establecer el nuevo xROI
                    if (((bndRec.X + bndRec.Width / 2) - Rectangulo2.Width / 2) >= 0) Rectangulo2.X = (bndRec.X + bndRec.Width / 2) - Rectangulo2.Width / 2;
                    else Rectangulo2.X = 0;
                    //Establecer el nuevo yROI
                    if (((bndRec.Y + bndRec.Height / 2) - Rectangulo2.Height / 2) >= 0) Rectangulo2.Y = (bndRec.Y + bndRec.Height / 2) - Rectangulo2.Height / 2;
                    else Rectangulo2.Y = 0;
                    //Si no está reducida la región al mínimo reducirla (focalizar la region)
                    if (Rectangulo2.Width > minwROI)
                    {
                        Rectangulo2.Width = Rectangulo2.Width - fwROI;
                        Rectangulo2.Height = Rectangulo2.Height - fhROI;
                        
                    }
                    //el valor encontrado es correcto
                    else
                    {
                        correcto2.Fill = new System.Windows.Media.SolidColorBrush(ColorMarcadorActivado);
                        
                    }
                }
            }
            //Ampliar region cuando no se encuentra ningun contorno (buscar el marcador)
            if (seq != null && seq.Ptr.ToInt64() == 0)
            {
                correcto2.Fill = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);
                if (Rectangulo2.Width < 640)
                {
                    if (xROI - (bwROI / 2) >= 0) Rectangulo2.X = Rectangulo2.X - (bwROI / 2);
                    else Rectangulo2.X = 0;
                    Rectangulo2.Width = Rectangulo2.Width + bwROI;
                }

                if (Rectangulo2.Height < 480)
                {
                    if (Rectangulo2.Y - (bhROI / 2) >= 0) Rectangulo2.Y = Rectangulo2.Y - (bhROI / 2);
                    else Rectangulo2.Y = 0;
                    Rectangulo2.Height = Rectangulo2.Height + bhROI;
                }
            }
        }
        /// <summary>
        /// Localización del centro de gravedad del blob blanco de mayor tamaño 
        /// en el seguimiento del marcador 3
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void RecDetection3(Image<Gray, byte> img)
        {
            Image<Gray, Byte> imgForContour = new Image<Gray, byte>(img.Width, img.Height);
            CvInvoke.cvCopy(img, imgForContour, System.IntPtr.Zero);
            IntPtr storage = CvInvoke.cvCreateMemStorage(0);
            IntPtr contour = new IntPtr(); //Puntero al primer contorno encontrado
            //Encontrar Contorno           
            int numContornos = CvInvoke.cvFindContours(
                imgForContour,
                storage,
                ref contour,
                System.Runtime.InteropServices.Marshal.SizeOf(typeof(MCvContour)),
                Emgu.CV.CvEnum.RETR_TYPE.CV_RETR_EXTERNAL,
                Emgu.CV.CvEnum.CHAIN_APPROX_METHOD.CV_CHAIN_APPROX_NONE, new System.Drawing.Point(Rectangulo3.X, Rectangulo3.Y));
            //Es necesario un punto seq para que lo utilice el método cvBoundingRect
            Seq<System.Drawing.Point> seq = new Seq<System.Drawing.Point>(contour, null);
            for (; seq != null && seq.Ptr.ToInt64() != 0; seq = seq.HNext)
            {
                //Crear rectángulo en el contorno encontrado             
                System.Drawing.Rectangle bndRec = CvInvoke.cvBoundingRect(contour, 1);
                //Calcular el pixel donde se encuentra el centro del contorno calculado                        
                int result = ((bndRec.Y + despY + bndRec.Width / 2) * 640) + (bndRec.X + despX + bndRec.Width / 2);
                //Garantizar que en el momento del mapeo no se salgan de los límites de la matriz
                if (result < (640 * 480))
                {
                    //Localizar el punto en el espacio esqueleto que corresponde a ese pixel
                    PuntoEnSkeleto = this.sensor.CoordinateMapper.MapDepthPointToSkeletonPoint(this.sensor.DepthStream.Format, mappedColorLocations[result]);
                    if (KinectSensor.IsKnownPoint(PuntoEnSkeleto))
                    {
                        guardarposicion(4, PuntoEnSkeleto);
                        m3x.Text = PuntoEnSkeleto.X.ToString();
                        m3y.Text = PuntoEnSkeleto.Y.ToString();
                        m3z.Text = PuntoEnSkeleto.Z.ToString();
                    }
                    /*Establezco el nuevo ROI */
                    //Establecer el nuevo xROI
                    if (((bndRec.X + bndRec.Width / 2) - Rectangulo3.Width / 2) >= 0) Rectangulo3.X = (bndRec.X + bndRec.Width / 2) - Rectangulo3.Width / 2;
                    else Rectangulo3.X = 0;
                    //Establecer el nuevo yROI
                    if (((bndRec.Y + bndRec.Height / 2) - Rectangulo3.Height / 2) >= 0) Rectangulo3.Y = (bndRec.Y + bndRec.Height / 2) - Rectangulo3.Height / 2;
                    else Rectangulo3.Y = 0;
                    //Si no está reducida la región al mínimo reducirla (focalizar la region)
                    if (Rectangulo3.Width > minwROI)
                    {
                        Rectangulo3.Width = Rectangulo3.Width - fwROI;
                        Rectangulo3.Height = Rectangulo3.Height - fhROI;
                        
                    }
                    //el valor encontrado es correcto
                    else
                    {
                        correcto3.Fill = new System.Windows.Media.SolidColorBrush(ColorMarcadorActivado);
                        

                    }
                }
            }
            //Ampliar region cuando no se encuentra ningun contorno (buscar el marcador)
            if (seq != null && seq.Ptr.ToInt64() == 0)
            {
                correcto3.Fill = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);
                if (Rectangulo3.Width < 640)
                {
                    if (xROI - (bwROI / 2) >= 0) Rectangulo3.X = Rectangulo3.X - (bwROI / 2);
                    else Rectangulo3.X = 0;
                    Rectangulo3.Width = Rectangulo3.Width + bwROI;
                }

                if (Rectangulo3.Height < 480)
                {
                    if (Rectangulo3.Y - (bhROI / 2) >= 0) Rectangulo3.Y = Rectangulo3.Y - (bhROI / 2);
                    else Rectangulo3.Y = 0;
                    Rectangulo3.Height = Rectangulo3.Height + bhROI;
                }
            }
        }
        /// <summary>
        /// Funcion que guardar una posicion actual  
        /// </summary>        
        /// <param name="elemento">mano o herramienta a guardar</param>
        /// <param name="posicion">punto en esqueleto que contiene la posicion a almacenar</param>
        private void guardarposicion(int elemento, SkeletonPoint posicion)
        {                      
            AlmacenaPuntos.SetPosicion(elemento, 0, posicion.X);
            AlmacenaPuntos.SetPosicion(elemento, 1, posicion.Y);
            AlmacenaPuntos.SetPosicion(elemento, 2, posicion.Z);
        }
        #endregion
#region Envío de posicion de marcadores
        const int MAX_BUFFER_SIZE = 150;
        bool cerrar = false;
        NetworkStream stream;
        TcpClient cliente;
        private void ConectarCliente(object sender, RoutedEventArgs e)
        {
            Int32 Port = 13000;
            string Host = "localhost";
            //Crear un cliente TCP
            cliente = new TcpClient(Host, Port);
            //Crear un Stream donde escribir
            stream = cliente.GetStream();
            envio.Enabled = true;
            envio.Start();
        }
        private void envio_Tick(object sender, EventArgs e)
        {
            string datosString = null;
            Byte[] datosByte = new byte[MAX_BUFFER_SIZE];

            if (stream.CanWrite && !cerrar)
            {
                for (int elemento = 0; elemento <= 4; elemento++)
                {
                    for (int coordenada = 0; coordenada <= 2; coordenada++)
                    {
                        datosString += AlmacenaPuntos.GetPosicion(elemento, coordenada).ToString();
                        datosString += ";";
                    }
                }
                datosByte = System.Text.Encoding.ASCII.GetBytes(datosString);
                stream.Write(datosByte, 0, datosByte.Length);
            }
        }   
        #endregion
#region Eventos de WPF
        /// <summary>
        /// Evento Click que inicia el seguimiento de los tres marcadores a la vez
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void IniciarSeguimiento_Click(object sender, RoutedEventArgs e)
        {
            marcador.Stop();
            marcador.Enabled = false;
            seguimiento = true;
            marcador1.Enabled = true;
            marcador1.Start();
            marcador2.Enabled = true;
            marcador2.Start();
            marcador3.Enabled = true;
            marcador3.Start();
        }
        private void PararSeguimiento_Click(object sender, RoutedEventArgs e)
        {
            marcador1.Stop();
            marcador1.Enabled = false;
            marcador2.Stop();
            marcador2.Enabled = false;
            marcador3.Stop();
            marcador3.Enabled = false;
            seguimiento = false;
            marcador.Enabled = true;
            marcador.Start();
        }
        /// <summary>
        /// Evento Click para cambiar los valores de Calibración del Seguimiento
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void CambiarValores_Click(object sender, RoutedEventArgs e)
        {
            despX = Convert.ToInt32(desplazarX.Text);
            despY = Convert.ToInt32(desplazarY.Text);
            xROI = 300;
            yROI = 220;            
            //tamaño máximo de ventana            
            switch (Convert.ToInt32(rt.Text))
            {
                case 1:
                    wROI = 60;
                    minwROI = 60;
                    hROI = 45;
                    minhROI = 45;
                    break;
                case 2:
                    wROI = 80;
                    minwROI = 80;
                    hROI = 60;
                    minhROI = 60;
                    break;
                case 3:
                    wROI = 100;
                    minwROI = 100;
                    hROI = 75;
                    minhROI = 75;
                    break;
                case 4:
                    wROI = 120;
                    minwROI = 120;
                    hROI = 90;
                    minhROI = 90;
                    break;
                case 5:
                    wROI = 140;
                    Rectangulo1.Width = 140;
                    Rectangulo2.Width = 140;
                    Rectangulo3.Width = 140;
                    minwROI = 140;
                    hROI = 105;
                    Rectangulo1.Height = 105;
                    Rectangulo2.Height = 105;
                    Rectangulo3.Height = 105;
                    minhROI = 105;
                    break;
                default:
                    rt.Text = "mal";
                    break;
            }
            switch (Convert.ToInt32(rb.Text))
            {
                case 1:
                    bwROI = 8;
                    bhROI = 6;
                    break;
                case 2:
                    bwROI = 16;
                    bhROI = 12;
                    break;
                case 3:
                    bwROI = 24;
                    bhROI = 18;
                    break;
                case 4:
                    bwROI = 40;
                    bhROI = 30;
                    break;
                case 5:
                    bwROI = 80;
                    bhROI = 60;
                    break;
                case 6: 
                    bwROI = 120;
                    bhROI = 90;
                    break;
                case 7: 
                    bwROI = 160;
                    bhROI = 120;
                    break;
                default:
                    rb.Text = "mal";
                    break;
            }
            switch (Convert.ToInt32(rf.Text))
            {
                case 1:
                    fwROI = 8;
                    fhROI = 6;
                    break;
                case 2:
                    fwROI = 16;
                    fhROI = 12;
                    break;
                case 3:
                    fwROI = 24;
                    fhROI = 18;
                    break;
                case 4:
                    fwROI = 40;
                    fhROI = 30;
                    break;
                case 5:
                    fwROI = 80;
                    fhROI = 60;
                    break;
                case 6:
                    fwROI = 120;
                    fhROI = 90;
                    break;
                case 7:
                    fwROI = 160;
                    fhROI = 120;
                    break;
                default:
                    rf.Text = "mal";
                    break;
            }
        }
        /// <summary>
        /// Evento ejecutado cuando se modifica la barra deslizante de los valores mínimos
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void trackBar_Lo_ValueChanged(object sender, EventArgs e)
        {
            //Actualiza el valor del Marcador activo            
            if (chValMac)
            {
                //Arrastra la barra del valor Máximo
                if (slHL.Value >= slHH.Value)
                    slHH.Value = slHL.Value + 1;
                if (slSL.Value >= slSH.Value)
                    slSH.Value = slSL.Value + 1;
                ValMar[MaAct, 0] = Convert.ToInt32(slHL.Value);
                ValMar[MaAct, 1] = Convert.ToInt32(slHH.Value);
                ValMar[MaAct, 2] = Convert.ToInt32(slSL.Value);
                ValMar[MaAct, 3] = Convert.ToInt32(slSH.Value);
            }
        }

        /// <summary>
        /// Evento ejecutado cuando se modifica la barra deslizante de los valores Máximos
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void trackBar_Hi_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            lock (MarcLock)
            {
                if (chValMac)
                {
                    //Arrastrasr las barras
                    if (slHH.Value <= slHL.Value)
                        slHL.Value = slHH.Value;
                    if (slSH.Value <= slSL.Value)
                        slSL.Value = slSH.Value;
                    //Cambia los valores de los marcadores
                    ValMar[MaAct, 0] = Convert.ToInt32(slHL.Value);
                    ValMar[MaAct, 1] = Convert.ToInt32(slHH.Value);
                    ValMar[MaAct, 2] = Convert.ToInt32(slSL.Value);
                    ValMar[MaAct, 3] = Convert.ToInt32(slSH.Value);
                }
            }
        }

        /// <summary>
        /// Evento ejecutado cuando se Clickea el Boton M1
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void SeleccionarMarcador1_Click(object sender, RoutedEventArgs e)
        {
            //Cambiar color botones
            SeleccionarMarcador1.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorActivado);
            SeleccionarMarcador2.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);
            SeleccionarMarcador3.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);
            //Cambiar estado Marcador Activo
            MaAct = 1;
            //Actualiza el valor de la Barra deslizante
            lock (MarcLock)
            {
                chValMac = false;
                slHL.Value = ValMar[MaAct, 0];
                slHH.Value = ValMar[MaAct, 1];
                slSL.Value = ValMar[MaAct, 2];
                slSH.Value = ValMar[MaAct, 3];
                chValMac = true;

            }
        }

        /// <summary>
        /// Evento ejecutado cuando se Clickea el Boton M2
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void SeleccionarMarcador2_Click(object sender, RoutedEventArgs e)
        {
            //Cambiar color Botones
            SeleccionarMarcador1.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);
            SeleccionarMarcador2.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorActivado);
            SeleccionarMarcador3.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);

            //Cambiar estado Marcador Activo
            MaAct = 2;
            //Actualiza el valor de la Barra deslizante
            lock (MarcLock)
            {
                chValMac = false;
                slHL.Value = ValMar[MaAct, 0];
                slHH.Value = ValMar[MaAct, 1];
                slSL.Value = ValMar[MaAct, 2];
                slSH.Value = ValMar[MaAct, 3];
                chValMac = true;
            }
        }
        /// <summary>
        /// Evento ejecutado cuando se Clickea el Boton M3
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void SeleccionarMarcador3_Click(object sender, RoutedEventArgs e)
        {
            //Cambiar color Botones
            SeleccionarMarcador1.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);
            SeleccionarMarcador2.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);
            SeleccionarMarcador3.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorActivado);
            //Cambiar estado Marcador Activo
            MaAct = 3;
            //Actualiza el valor de la Barra deslizante
            lock (MarcLock)
            {
                chValMac = false;
                slHL.Value = ValMar[MaAct, 0];
                slHH.Value = ValMar[MaAct, 1];
                slSL.Value = ValMar[MaAct, 2];
                slSH.Value = ValMar[MaAct, 3];
                chValMac = true;
            }
        }
        /// <summary>
        /// Evento ejecutado cuando se Clickea el Boton Nuevo. Pone los valores de los marcadores a Cero
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void NuevosMarcadores_Click(object sender, RoutedEventArgs e)
        {
            MaAct = 0;  //Desactiva los Marcadores
            //Cambiar color de los Botones
            SeleccionarMarcador1.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);
            SeleccionarMarcador2.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);
            SeleccionarMarcador3.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);
            //Cambiar el valor de los Marcadores a 0
            lock (MarcLock)
            {
                for (int i = 1; i < NUM_MARCADORES; i++)
                {
                    ValMar[i, 0] = 0;
                    ValMar[i, 1] = 1;
                    ValMar[i, 2] = 0;
                    ValMar[i, 3] = 1;
                }
            }
        }
        /// <summary>
        /// Evento ejecutado cuando se Clickea el Boton Cargar. Se Establecen los valores de los marcadores
        /// con la información guardada en un archivo
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void CargarMarcadores_Click(object sender, RoutedEventArgs e)
        {
            MaAct = 0;  //Desactiva los Marcadores
            //Cambiar color de los Botones
            SeleccionarMarcador1.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);
            SeleccionarMarcador2.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);
            SeleccionarMarcador3.Background = new System.Windows.Media.SolidColorBrush(ColorMarcadorDesactivado);
            //Abrir archivo para lectura
            string ruta = @"C:\Proyecto\Record\colores\" + NombreCargarColores.Text + ".txt";
            System.IO.StreamReader objReader = new System.IO.StreamReader(ruta);
            string leido = objReader.ReadLine();
            //Dividir mensaje
            string[] words;
            char[] delimiterChars = { ';' };
            words = leido.Split(delimiterChars);
            int contador = 0;
            //Cambiar el valor de los Marcadores
            lock (MarcLock)
            {
                for (int i = 1; i < NUM_MARCADORES; i++)
                {
                    for (int j = 0; j < NUM_PARAM; j++)
                    {
                        ValMar[i, j] = int.Parse(words[contador]);
                        contador++;
                    }
                }
            }
        }
        /// <summary>
        /// Evento Click que graba los parámetros actuales de los tres marcadores
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void GuardarMarcadores_Click(object sender, RoutedEventArgs e)
        {
            string ruta = @"C:\Proyecto\Record\colores\" + NombreGuardarColores.Text + ".txt";
            string linea = null;
            for (int i = 1; i < NUM_MARCADORES; i++)
            {
                for (int j = 0; j < NUM_PARAM; j++)
                {
                    linea += ValMar[i, j].ToString();
                    linea += ";";
                }
            }

            using (System.IO.StreamWriter file = new System.IO.StreamWriter(ruta, true))
            {
                file.WriteLine(linea);
            }
            NombreCargarColores.Text = NombreGuardarColores.Text;
        }
        /// <summary>
        /// Evento Click para cambiar la imagen secundaria 1 a Principal
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void ImgSec1toPrincipal_Click(object sender, RoutedEventArgs e)
        {
            if (seguimiento)
            {
                if (imagenColor1 == 0) imagenColor1 = 1;
                else if (imagenColor1 == 1) imagenColor1 = 0;
                if (imagenColor2 == 0) imagenColor2 = 1;
                else if (imagenColor2 == 1) imagenColor2 = 0;
                if (imagenColor3 == 0) imagenColor3 = 1;
                else if (imagenColor3 == 1) imagenColor3 = 0;
            }
            else
            {
                if (imagenColor == 0) imagenColor = 1;
                else if (imagenColor == 1) imagenColor = 0;
                if (imagenBinaria == 0) imagenBinaria = 1;
                else if (imagenBinaria == 1) imagenBinaria = 0;
                if (imagenErosionada == 0) imagenErosionada = 1;
                else if (imagenErosionada == 1) imagenErosionada = 0;
            }            
        }
        /// <summary>
        /// Evento Click para cambiar la imagen secundaria 2 a Principal
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void ImgSec2toPrincipal_Click(object sender, RoutedEventArgs e)
        {
            if (seguimiento)
            {
                if (imagenColor1 == 0) imagenColor1 = 2;
                else if (imagenColor1 == 2) imagenColor1 = 0;
                if (imagenColor2 == 0) imagenColor2 = 2;
                else if (imagenColor2 == 2) imagenColor2 = 0;
                if (imagenColor3 == 0) imagenColor3 = 2;
                else if (imagenColor3 == 2) imagenColor3 = 0;
            }
            else
            {
                if (imagenColor == 0) imagenColor = 2;
                else if (imagenColor == 2) imagenColor = 0;
                if (imagenBinaria == 0) imagenBinaria = 2;
                else if (imagenBinaria == 2) imagenBinaria = 0;
                if (imagenErosionada == 0) imagenErosionada = 2;
                else if (imagenErosionada == 2) imagenErosionada = 0;
            }
        }      
        /// <summary>
        /// Evento Click para iniciar la grabación de la trayectoria que sigue el Marcador Activo
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void IniciarGrabacion_Click(object sender, RoutedEventArgs e)
        {
            string nombre = nombreArchivo.Text;
            rutaTrayectoria = @"C:\Proyecto\Record\trayectorias\" + nombre + ".txt";
            string primeralinea = "MX;MY;MSkX;MSkY;MSkZ";

            using (System.IO.StreamWriter file = new System.IO.StreamWriter(rutaTrayectoria, true))
            {
                file.WriteLine(primeralinea);
            }
            lock (this)
            {
                grabar = true;
            }
        }
        /// <summary>
        /// Evento Click para detener la grabación de la trayectoria que sigue el Marcador Activo
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void PararGrabacion_Click(object sender, RoutedEventArgs e)
        {
            lock (this)
            {
                grabar = false;
            }
        }
        /// <summary>
        /// Evento Click para cambiar el Angulo de rotación del sensor Kienct
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void cambiarAngulo_Click(object sender, RoutedEventArgs e)
        {
            int gradonuevo = int.Parse(nuevoAngulo.Text);
            if (gradonuevo < this.sensor.MaxElevationAngle && gradonuevo > this.sensor.MinElevationAngle)
            {
                this.sensor.ElevationAngle = gradonuevo;
                anguloActual.Text = "Ángulo actual: " + this.sensor.ElevationAngle;
            }
            else
            {
                anguloActual.Text = "No valido, entre -27 y 27";
            }
        }
        /// <summary>
        /// Evento Click para conocer los grados Actuales rotados del sensor kinect
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void gradosActuales_Click(object sender, RoutedEventArgs e)
        {
            anguloActual.Text = "Ángulo actual: " + this.sensor.ElevationAngle;
        }
        #endregion
#region Guardar Configuración
        bool permisoColor = true;
        bool permisoOpcion = false;
        private void guardarConfiguracion_Click(object sender, RoutedEventArgs e)
        {
            string ruta = @"C:\Proyecto\Record\configuracion\" + NombreGuardarConfiguracion.Text + ".txt";
            string linea = null;

            if (permisoColor)
            {
                for (int i = 1; i < NUM_MARCADORES; i++)
                {
                    for (int j = 0; j < NUM_PARAM; j++)
                    {
                        linea += ValMar[i, j].ToString();
                        linea += ";";
                    }
                }
            }
                //valores por defecto
            else linea += "0;10;137;255;124;149;67;255;21;30;134;255;";
            if (permisoOpcion)
            {
                linea += desplazarX.Text + ";" + desplazarY.Text + ";" + rt.Text + ";" + rb.Text + ";" + rf.Text + ";";

            }
            else //valores por defecto
            {
                linea += "0;3;140;105;120;90;80;60;";
            }
            using (System.IO.StreamWriter file = new System.IO.StreamWriter(ruta, true))
            {
                file.WriteLine(linea);
            }
        }

        private void permisoColores_Checked(object sender, RoutedEventArgs e)
        {
            permisoColor = true;
        }

        private void permisoColores_Unchecked(object sender, RoutedEventArgs e)
        {
            permisoColor = false;
        }

        private void permisoOpciones_Unchecked(object sender, RoutedEventArgs e)
        {
            permisoOpcion = false;
        }

        private void permisoOpciones_Checked(object sender, RoutedEventArgs e)
        {
            permisoOpcion = true;
        }
        #endregion
#region Conversion De Imagenes
        /// <summary>
        /// Conversion del Color Frame de kinect en formato BitmapSource a Bitmap
        /// </summary>
        /// <param name="colorFrame">Imagen en formato ColorImageFrame a covertir</param>        
        private static Bitmap ColorImageFrameToBitmap(ColorImageFrame colorFrame)
        {
            int ancho = 640;
            int alto = 480;

            Bitmap bitmapFrame = new Bitmap(ancho, alto);
            byte[] pixelBuffer = new byte[colorFrame.PixelDataLength];
            colorFrame.CopyPixelDataTo(pixelBuffer);
            //bitmapFrame = new Bitmap(ancho, alto, PixelFormat.Format32bppRgb);                                    
            
            BitmapData bitmapData = bitmapFrame.LockBits(new Rectangle(0, 0,
                                             colorFrame.Width, colorFrame.Height),
            ImageLockMode.WriteOnly, bitmapFrame.PixelFormat);

            IntPtr intPointer = bitmapData.Scan0;
            System.Runtime.InteropServices.Marshal.Copy(pixelBuffer, 0, intPointer, colorFrame.PixelDataLength);

            bitmapFrame.UnlockBits(bitmapData);

            return bitmapFrame;
        }
        #endregion                                
    }
}
